@@ -1,24 +1,39 @@
-//
-// Copyright(c) 2015 Gabi Melman.
-// Distributed under the MIT License (http://opensource.org/licenses/MIT)
-//
+/*************************************************************************/
+/* spdlog - an extremely fast and easy to use c++11 logging library.     */
+/* Copyright (c) 2014 Gabi Melman.                                       */
+/*                                                                       */
+/* Permission is hereby granted, free of charge, to any person obtaining */
+/* a copy of this software and associated documentation files (the       */
+/* "Software"), to deal in the Software without restriction, including   */
+/* without limitation the rights to use, copy, modify, merge, publish,   */
+/* distribute, sublicense, and/or sell copies of the Software, and to    */
+/* permit persons to whom the Software is furnished to do so, subject to */
+/* the following conditions:                                             */
+/*                                                                       */
+/* The above copyright notice and this permission notice shall be        */
+/* included in all copies or substantial portions of the Software.       */
+/*                                                                       */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
+/*************************************************************************/
 
 #pragma once
 
-#include "spdlog/formatter.h"
-#include "spdlog/details/log_msg.h"
-#include "spdlog/details/os.h"
-#include "spdlog/fmt/fmt.h"
-
-#include <chrono>
-#include <ctime>
-#include <memory>
-#include <mutex>
 #include <string>
-#include <thread>
-#include <utility>
+#include <chrono>
+#include <memory>
 #include <vector>
-#include <array>
+#include <thread>
+
+
+#include "../formatter.h"
+#include "./log_msg.h"
+#include "./os.h"
 
 namespace spdlog
 {
@@ -27,8 +42,7 @@ namespace details
 class flag_formatter
 {
 public:
-    virtual ~flag_formatter()
-    {}
+    virtual ~flag_formatter() {}
     virtual void format(details::log_msg& msg, const std::tm& tm_time) = 0;
 };
 
@@ -37,17 +51,17 @@ public:
 ///////////////////////////////////////////////////////////////////////
 namespace
 {
-class name_formatter:public flag_formatter
+class name_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
-        msg.formatted << *msg.logger_name;
+        msg.formatted << msg.logger_name;
     }
 };
 }
 
 // log level appender
-class level_formatter:public flag_formatter
+class level_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -56,7 +70,7 @@ class level_formatter:public flag_formatter
 };
 
 // short log level appender
-class short_level_formatter:public flag_formatter
+class short_level_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -80,7 +94,7 @@ static int to12h(const tm& t)
 
 //Abbreviated weekday name
 static const std::string days[] { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
-class a_formatter:public flag_formatter
+class a_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -90,7 +104,7 @@ class a_formatter:public flag_formatter
 
 //Full weekday name
 static const std::string full_days[] { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };
-class A_formatter:public flag_formatter
+class A_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -100,17 +114,17 @@ class A_formatter:public flag_formatter
 
 //Abbreviated month
 static const std::string  months[] { "Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec" };
-class b_formatter:public flag_formatter
+class b_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << months[tm_time.tm_mon];
+        msg.formatted<< months[tm_time.tm_mon];
     }
 };
 
 //Full month name
 static const std::string full_months[] { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
-class B_formatter:public flag_formatter
+class B_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -135,7 +149,7 @@ static fmt::MemoryWriter& pad_n_join(fmt::MemoryWriter& w, int v1, int v2, int v
 
 
 //Date and time representation (Thu Aug 23 15:35:46 2014)
-class c_formatter SPDLOG_FINAL:public flag_formatter
+class c_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -146,7 +160,7 @@ class c_formatter SPDLOG_FINAL:public flag_formatter
 
 
 // year - 2 digit
-class C_formatter SPDLOG_FINAL:public flag_formatter
+class C_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -157,7 +171,7 @@ class C_formatter SPDLOG_FINAL:public flag_formatter
 
 
 // Short MM/DD/YY date, equivalent to %m/%d/%y 08/23/01
-class D_formatter SPDLOG_FINAL:public flag_formatter
+class D_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -167,7 +181,7 @@ class D_formatter SPDLOG_FINAL:public flag_formatter
 
 
 // year - 4 digit
-class Y_formatter SPDLOG_FINAL:public flag_formatter
+class Y_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -176,7 +190,7 @@ class Y_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // month 1-12
-class m_formatter SPDLOG_FINAL:public flag_formatter
+class m_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -185,7 +199,7 @@ class m_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // day of month 1-31
-class d_formatter SPDLOG_FINAL:public flag_formatter
+class d_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -194,7 +208,7 @@ class d_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // hours in 24 format  0-23
-class H_formatter SPDLOG_FINAL:public flag_formatter
+class H_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -203,7 +217,7 @@ class H_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // hours in 12 format  1-12
-class I_formatter SPDLOG_FINAL:public flag_formatter
+class I_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -211,8 +225,8 @@ class I_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
-// minutes 0-59
-class M_formatter SPDLOG_FINAL:public flag_formatter
+// ninutes 0-59
+class M_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -221,7 +235,7 @@ class M_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // seconds 0-59
-class S_formatter SPDLOG_FINAL:public flag_formatter
+class S_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -230,7 +244,7 @@ class S_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // milliseconds
-class e_formatter SPDLOG_FINAL:public flag_formatter
+class e_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -241,7 +255,7 @@ class e_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // microseconds
-class f_formatter SPDLOG_FINAL:public flag_formatter
+class f_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -251,19 +265,8 @@ class f_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
-// nanoseconds
-class F_formatter SPDLOG_FINAL:public flag_formatter
-{
-    void format(details::log_msg& msg, const std::tm&) override
-    {
-        auto duration = msg.time.time_since_epoch();
-        auto ns = std::chrono::duration_cast<std::chrono::nanoseconds>(duration).count() % 1000000000;
-        msg.formatted << fmt::pad(static_cast<int>(ns), 9, '0');
-    }
-};
-
 // AM/PM
-class p_formatter SPDLOG_FINAL:public flag_formatter
+class p_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -273,7 +276,7 @@ class p_formatter SPDLOG_FINAL:public flag_formatter
 
 
 // 12 hour clock 02:55:02 pm
-class r_formatter SPDLOG_FINAL:public flag_formatter
+class r_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -282,7 +285,7 @@ class r_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // 24-hour HH:MM time, equivalent to %H:%M
-class R_formatter SPDLOG_FINAL:public flag_formatter
+class R_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -291,7 +294,7 @@ class R_formatter SPDLOG_FINAL:public flag_formatter
 };
 
 // ISO 8601 time format (HH:MM:SS), equivalent to %H:%M:%S
-class T_formatter SPDLOG_FINAL:public flag_formatter
+class T_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -299,14 +302,14 @@ class T_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
+
 // ISO 8601 offset from UTC in timezone (+-HH:MM)
-class z_formatter SPDLOG_FINAL:public flag_formatter
+class z_formatter :public flag_formatter
 {
 public:
     const std::chrono::seconds cache_refresh = std::chrono::seconds(5);
 
-    z_formatter():_last_update(std::chrono::seconds(0)), _offset_minutes(0)
-    {}
+    z_formatter() :_last_update(std::chrono::seconds(0)) {}
     z_formatter(const z_formatter&) = delete;
     z_formatter& operator=(const z_formatter&) = delete;
 
@@ -319,20 +322,10 @@ public:
         // it is very fast (already stored in tm.tm_gmtoff)
         int total_minutes = os::utc_minutes_offset(tm_time);
 #endif
-        bool is_negative = total_minutes < 0;
-        char sign;
-        if (is_negative)
-        {
-            total_minutes = -total_minutes;
-            sign = '-';
-        }
-        else
-        {
-            sign = '+';
-        }
 
         int h = total_minutes / 60;
         int m = total_minutes % 60;
+        char sign = h >= 0 ? '+' : '-';
         msg.formatted << sign;
         pad_n_join(msg.formatted, h, m, ':');
     }
@@ -356,8 +349,8 @@ private:
 
 
 
-// Thread id
-class t_formatter SPDLOG_FINAL:public flag_formatter
+//Thread id
+class t_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -365,17 +358,8 @@ class t_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
-// Current pid
-class pid_formatter SPDLOG_FINAL:public flag_formatter
-{
-    void format(details::log_msg& msg, const std::tm&) override
-    {
-        msg.formatted << details::os::pid();
-    }
-};
 
-
-class v_formatter SPDLOG_FINAL:public flag_formatter
+class v_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -383,10 +367,10 @@ class v_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
-class ch_formatter SPDLOG_FINAL:public flag_formatter
+class ch_formatter :public flag_formatter
 {
 public:
-    explicit ch_formatter(char ch): _ch(ch)
+    explicit ch_formatter(char ch) : _ch(ch)
     {}
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -398,7 +382,7 @@ private:
 
 
 //aggregate user chars to display as is
-class aggregate_formatter SPDLOG_FINAL:public flag_formatter
+class aggregate_formatter :public flag_formatter
 {
 public:
     aggregate_formatter()
@@ -417,7 +401,7 @@ private:
 
 // Full info formatter
 // pattern: [%Y-%m-%d %H:%M:%S.%e] [%n] [%l] %v
-class full_formatter SPDLOG_FINAL:public flag_formatter
+class full_formatter :public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -448,13 +432,13 @@ class full_formatter SPDLOG_FINAL:public flag_formatter
                       << fmt::pad(static_cast<unsigned int>(tm_time.tm_sec), 2, '0') << '.'
                       << fmt::pad(static_cast<unsigned int>(millis), 3, '0') << "] ";
 
-        //no datetime needed
+//no datetime needed
 #else
         (void)tm_time;
 #endif
 
 #ifndef SPDLOG_NO_NAME
-        msg.formatted << '[' << *msg.logger_name << "] ";
+        msg.formatted << '[' << msg.logger_name << "] ";
 #endif
 
         msg.formatted << '[' << level::to_str(msg.level) << "] ";
@@ -462,15 +446,12 @@ class full_formatter SPDLOG_FINAL:public flag_formatter
     }
 };
 
-
-
 }
 }
 ///////////////////////////////////////////////////////////////////////////////
 // pattern_formatter inline impl
 ///////////////////////////////////////////////////////////////////////////////
-inline spdlog::pattern_formatter::pattern_formatter(const std::string& pattern, pattern_time_type pattern_time)
-    : _pattern_time(pattern_time)
+inline spdlog::pattern_formatter::pattern_formatter(const std::string& pattern)
 {
     compile_pattern(pattern);
 }
@@ -521,101 +502,98 @@ inline void spdlog::pattern_formatter::handle_flag(char flag)
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::short_level_formatter()));
         break;
 
-    case('t'):
+    case('t') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::t_formatter()));
         break;
 
-    case('v'):
+    case('v') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::v_formatter()));
         break;
 
-    case('a'):
+    case('a') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::a_formatter()));
         break;
 
-    case('A'):
+    case('A') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::A_formatter()));
         break;
 
-    case('b'):
-    case('h'):
+    case('b') :
+    case('h') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::b_formatter()));
         break;
 
-    case('B'):
+    case('B') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::B_formatter()));
         break;
-    case('c'):
+    case('c') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::c_formatter()));
         break;
 
-    case('C'):
+    case('C') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::C_formatter()));
         break;
 
-    case('Y'):
+    case('Y') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::Y_formatter()));
         break;
 
-    case('D'):
-    case('x'):
+    case('D') :
+    case('x') :
 
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::D_formatter()));
         break;
 
-    case('m'):
+    case('m') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::m_formatter()));
         break;
 
-    case('d'):
+    case('d') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::d_formatter()));
         break;
 
-    case('H'):
+    case('H') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::H_formatter()));
         break;
 
-    case('I'):
+    case('I') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::I_formatter()));
         break;
 
-    case('M'):
+    case('M') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::M_formatter()));
         break;
 
-    case('S'):
+    case('S') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::S_formatter()));
         break;
 
-    case('e'):
+    case('e') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::e_formatter()));
         break;
 
-    case('f'):
+    case('f') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::f_formatter()));
         break;
-    case('F'):
-        _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::F_formatter()));
-        break;
 
-    case('p'):
+    case('p') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::p_formatter()));
         break;
 
-    case('r'):
+    case('r') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::r_formatter()));
         break;
 
-    case('R'):
+    case('R') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::R_formatter()));
         break;
 
-    case('T'):
-    case('X'):
+    case('T') :
+    case('X') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::T_formatter()));
         break;
 
-    case('z'):
+    case('z') :
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::z_formatter()));
         break;
 
@@ -623,43 +601,28 @@ inline void spdlog::pattern_formatter::handle_flag(char flag)
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::full_formatter()));
         break;
 
-    case ('P'):
-        _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::pid_formatter()));
-        break;
-
-#if defined(SPDLOG_ENABLE_MESSAGE_COUNTER)
-    case ('i'):
-        _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::i_formatter()));
-        break;
-#endif
-
-    default: //Unknown flag appears as is
+    default: //Unkown flag appears as is
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::ch_formatter('%')));
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::ch_formatter(flag)));
         break;
     }
 }
 
-inline std::tm spdlog::pattern_formatter::get_time(details::log_msg& msg)
-{
-    if (_pattern_time == pattern_time_type::local)
-        return details::os::localtime(log_clock::to_time_t(msg.time));
-    else
-        return details::os::gmtime(log_clock::to_time_t(msg.time));
-}
 
 inline void spdlog::pattern_formatter::format(details::log_msg& msg)
 {
-
-#ifndef SPDLOG_NO_DATETIME
-    auto tm_time = get_time(msg);
-#else
-    std::tm tm_time;
-#endif
-    for (auto &f : _formatters)
+    try
     {
-        f->format(msg, tm_time);
+        auto tm_time = details::os::localtime(log_clock::to_time_t(msg.time));
+        for (auto &f : _formatters)
+        {
+            f->format(msg, tm_time);
+        }
+        //write eol
+        msg.formatted << details::os::eol();
+    }
+    catch(const fmt::FormatError& e)
+    {
+        throw spdlog_ex(fmt::format("formatting error while processing format string: {}", e.what()));
     }
-    //write eol
-    msg.formatted.write(details::os::eol, details::os::eol_size);
 }
